import sys
import os
import yaml
from pathlib import Path

from resumaker.utils import read_file_txt, render_yaml_txt, is_config_valid


def get_base_dir():
    path = Path(__file__).resolve().parent
    return path


CONFIG_FILENAME = "config.yml"
default_configs = {
    "RESUME_FILENAME": ["resume.yml", "resume.yaml"],
"ORDER" : ["summary", "education", "skills", "work_experience", "projects", "links"],
    "BASE_DIR": get_base_dir(),
}
BASE_DIR = get_base_dir()


def get_config():
    """
    Return config data by overwriting default_configs
    by reading config.yml if exists else default_configs
    """
    try:
        user_config_txt = read_file_txt(CONFIG_FILENAME)
    except FileNotFoundError:
        return default_configs
    else:
        return manage_user_config_txt(user_config_txt)


def manage_user_config_txt(user_config_txt):
    """
    if config_obj is valid, merge it with default_configs
    and return else return default_configs
    """
    user_config_obj = render_yaml_txt(user_config_txt)
    is_valid, err_msg = is_config_valid(user_config_obj)
    if is_valid:
        return merge_user_config_with_defaults(user_config_obj)
    else:
        print(f"Warning: {err_msg}. Using default configs.")
        return default_configs


def merge_user_config_with_defaults(user_config_obj):
    """
    Overwrite default_configs with user_config_obj.
    return default_config
    """
    for config in user_config_obj:
        if config in default_configs:
            user_config_value = user_config_obj[config]
            default_configs[config] = (
                user_config_value
                if type(user_config_value) == list
                else [user_config_value]
            )
        else:
            print(f"{config} is not supported!")
    return default_configs


config = get_config()



import argparse
import json
import sys

from resumaker.config import config
from resumaker.utils import is_resume_obj_valid, common_fields
from resumaker.render_resume import *
from resumaker.templates.t1.resume import Resume
from resumaker.templates.t1.template import ResumeTemplate
from resumaker.__about__ import __version__


def main():
    parser = get_parser()

    args = parser.parse_args()

    resume_filenames = get_resume_filenames(args.filenames)

    # check if files exist
    complete_resume_obj, filename, errors = get_resume_obj(resume_filenames)
    if errors:
        for error in errors:
            print(error)
        sys.exit(1)
    

    # check validity (schema) of resume common fields
    is_valid, err_msg = is_resume_obj_valid(complete_resume_obj, filename)
    if not is_valid:
        sys.exit(err_msg)

    common_sections = {
        "name": complete_resume_obj.get("name"),
        "location": complete_resume_obj.get("location"),
        "contact": complete_resume_obj.get("contact"),
        "education": complete_resume_obj.get("education"),
        "links": complete_resume_obj.get("links"),
    }

    all_targets = get_targets(args.target, complete_resume_obj, common_sections)

    template = ResumeTemplate("t1")

    for target in all_targets:
        resume = Resume(
            target,
            template,
            order=config.get("ORDER"),
        )
        resume.build(build_name(target))


def build_name(target):
    user_name = target["name"].replace(" ", "").lower()
    target_name = target["target_name"]
    return f"{user_name}-{target_name}"


def get_resume_filenames(filenames_arg):
    resume_filenames = config["RESUME_FILENAME"]

    if filenames_arg:
        resume_filenames = filenames_arg
        if type(resume_filenames) != list:
            resume_filenames = [filenames_arg]

    return resume_filenames


def get_targets(target_args, complete_resume_obj, common_sections):
    user_specified_target = target_args if target_args else None
    user_given_targets = [
        key for key in complete_resume_obj.keys() if key not in common_fields
    ]

    all_targets = []
    if user_specified_target:
        if user_specified_target in user_given_targets:
            all_targets.append(
                complete_resume_obj[user_specified_target]
                | {"target_name": user_specified_target}
                | common_sections
            )
        else:
            sys.exit("Target is not in the resume")
    else:
        for target in user_given_targets:
            all_targets.append(
                complete_resume_obj[target] | {"target_name": target} | common_sections
            )

    return all_targets


def get_parser():
    parser = argparse.ArgumentParser(
        prog="resumaker",
        description="Build multi-profile ATS friendly resume from a single YAML file.",
        epilog="Thank you for using resumaker.\n"
        "To contribute, please visit https://github.com/sujaudd1n/resumaker.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "-f",
        "--filename",
        metavar="filename",
        dest="filenames",
        help="Filename of resume yaml file.",
    )
    parser.add_argument(
        "-t",
        "--target",
        metavar="target",
        dest="target",
        help="Select target such as devops or AI in your resume",
    )
    parser.add_argument(
        "--version", action="version", version=f"%(prog)s {__version__}"
    )
    return parser


if __name__ == "__main__":
    main()



import sys
import os
import yaml

common_fields = {
    "name": str,
    "location": str,
    "contact": dict,
    "education": list,
    "links": dict,
    "skills": dict,
}


def read_file_txt(filename):
    """Read filename and return its text"""
    with open(filename) as f:
        return f.read()


def render_yaml_txt(yaml_txt):
    "Render and return yaml_txt"
    try:
        return yaml.safe_load(yaml_txt)
    except Exception as exc:
        sys.exit("Error: Coult not parse yaml: " + str(exc))


def is_resume_obj_valid(resume_obj, filename):
    def check(field, dtype, dtype_msg):
        assert resume_obj.get(field), f"{field} does not exist in {filename}"
        assert isinstance(
            resume_obj.get(field), dtype
        ), f"{field} should be {dtype_msg}"

    try:
        assert isinstance(resume_obj, dict), f"{filename} should be key-value pair"

        check("name", str, "a str")
        check("contact", dict, "a dictionary")
        check("education", list, "a list of dictionary")
        check("links", dict, "a dictionary")

        is_resume_contact_valid(resume_obj.get("contact"))
        is_resume_education_valid(resume_obj.get("education"))

    except AssertionError as exc:
        return False, str(exc)
    else:
        return True, None


def is_resume_contact_valid(contact):
    valid_keys = ["name", "location", "email", "phone", "linkedin", "github"]
    for key, val in contact.items():
        assert key in valid_keys, f"{key} is not valid for contact"
        assert type(val) == str, f"Value of contact.{key} should be a str"
    return True, None


def is_resume_education_valid(education_list):
    for education in education_list:
        assert type(education) == dict, "Education entry should be a dictionary"
        for key, val in education.items():
            if key == "achievements":
                assert (
                    type(education[key]) == list
                ), f"Value of education.<entry>.{key} should be a list"
                for achievement in education[key]:
                    assert (
                        type(achievement) == str
                    ), f"Value of education.<entry>.{key}.<item> should be a str"
            else:
                assert (
                    type(val) == str
                ), f"Value of education.<entry>.{key} should be a str"
    return True, None


def is_config_valid(config_obj):
    """
    config_obj has to be a dict.
    Each value in key: value has to be a str or list
    """
    try:
        assert isinstance(
            config_obj, dict
        ), "config.yml should be convertible to Python dict"
        for _, val in config_obj.items():
            assert isinstance(val, list) or isinstance(
                val, str
            ), "Value of config has to be str or list of str"
    except AssertionError as exc:
        return False, str(exc)
    return True, None



import pytest
from resumaker.templates.t1.template import *
from .tex_template import tex_template


class TestResumeTemplate:
    def test_template_import(self):
        resumeTemplate = ResumeTemplate("t1")
        template_dict = resumeTemplate.get_template_obj()
        assert type(template_dict) == dict


class TestContactTemplate:
    @pytest.fixture
    def contactObj(self):
        values = {
            "name": "test",
            "location": "location",
            "phone": "phone",
            "email": "email",
            "linkedin": "linkedin",
            "github": "github",
        }
        template = "$name$location$phone$email$linkedin$github"

        contactTemplate = ContactTemplate(template, values)
        return contactTemplate

    def test_render_tex(self, contactObj):
        tex = contactObj.render_tex()
        assert tex == "testlocationphoneemaillinkedingithub"


class TestSummaryTemplate:
    @pytest.fixture
    def summaryObj(self):
        values = {
            "title": "summary title",
            "text": "summary text",
        }
        template = "$title $text"

        summaryTemplate = SummaryTemplate(template, values)
        return summaryTemplate

    def test_render_tex(self, summaryObj):
        tex = summaryObj.render_tex()
        assert tex == "summary title summary text"


class TestEducationTemplate:
    @pytest.fixture
    def educationObj(self):
        values = [
            {
                "name": "CLGY",
                "location": "LOCX",
                "degree": "DEGZ",
                "duration": "DUR3",
                "achievements": ["A1", "A2", "A3"],
            },
            {
                "name": "CLGY2",
                "location": "LOCX2",
                "degree": "DEGZ2",
                "duration": "DUR32",
                "achievements": ["A12", "A22", "A32"],
            },
        ]
        template = {
            "complete": "S $all_educations E",
            "single": "$degree $duration $name $location $all_achievements",
            "achievement": "$achievement",
        }

        educationTemplate = EducationTemplate(template, values)
        return educationTemplate

    def test_render_tex(self, educationObj):
        tex = educationObj.render_tex()
        assert tex == (
            "S DEGZ DUR3 CLGY LOCX A1 A2 A3\nDEGZ2 DUR32 CLGY2 LOCX2 A12 A22 A32 E"
        )


class TestLinksTemplate:
    @pytest.fixture
    def linksObj(self):
        values = {
            "algorithms": [
                {
                    "name": "l1",
                    "url": "http://example.com/l1",
                    "url_text": "l1text",
                },
                {
                    "name": "l3",
                    "url": "http://example.com/l3",
                    "url_text": "l3text",
                },
            ],
            "open-source": [
                {
                    "name": "l2",
                    "url": "http://example.com/l2",
                    "url_text": "l2text",
                }
            ],
        }
        template = {
            "complete": "$all_links",
            "single": "$link_title $single_links",
            "single-link": "$link_url $link_url_text",
        }

        linksTemplate = LinksTemplate(template, values)
        return linksTemplate

    def test_render_tex(self, linksObj):
        tex = linksObj.render_tex()
        assert tex == (
            "Algorithms http://example.com/l1 l1text http://example.com/l3 l3text\n"
            "Open Source http://example.com/l2 l2text"
        )


class TestSkillsTemplate:
    @pytest.fixture
    def skillsObj(self):
        values = {
            "st1": ["s1", "s2"],
            "st2": ["s3"],
        }
        template = {
            "complete": "$all_skills",
            "single": "$skill_topic $skills_list",
        }

        skillsTemplate = SkillsTemplate(template, values)
        return skillsTemplate

    def test_render_tex(self, skillsObj):
        tex = skillsObj.render_tex()
        assert tex == ("St1 s1, s2\nSt2 s3")

    def test_format_skills_list(self, skillsObj):
        topic3 = [
            "WIDTH-1010",
            "WIDTH-1010",
            "WIDTH-1010",
            "WIDTH-1010",
            "WIDTH-1010",
            "WIDTH-1010",
            "WIDTH-1010",
            "WIDTH-1010",
            "WIDTH-1010",
            "WIDTH-1010",
            "WIDTH-1010",
            "WIDTH-1010",
        ]  # 12 total
        skills = skillsObj.format_skills_list(topic3)
        assert skills == (
            "WIDTH-1010, WIDTH-1010, WIDTH-1010, WIDTH-1010, "
            "WIDTH-1010, WIDTH-1010, WIDTH-1010, WIDTH-1010"
            " \\\\ &\n"
            "WIDTH-1010, WIDTH-1010, WIDTH-1010, WIDTH-1010"
        )


class TestWorkExperienceTemplate:
    @pytest.fixture
    def work_experience_obj(self):
        values = [
            {
                "company-name": "C1",
                "location": "L1",
                "role": "R1",
                "duration": "D1",
                "contributions": ["C1", "C2", "C3"],
            },
            {
                "company-name": "C2",
                "location": "L2",
                "role": "R2",
                "duration": "D2",
                "contributions": ["C4", "C5", "C6"],
            },
        ]
        template = {
            "complete": "$all_work_experiences",
            "single": "$company_name $location $role $duration $all_contributions",
            "single-contribution": "$contribution",
        }

        work_experience_template = WorkExperienceTemplate(template, values)
        return work_experience_template

    def test_render_tex(self, work_experience_obj):
        tex = work_experience_obj.render_tex()
        assert tex == ("C1 L1 R1 D1 C1\nC2\nC3\nC2 L2 R2 D2 C4\nC5\nC6")


class TestProjectTemplate:
    @pytest.fixture
    def project_obj(self):
        values = [
            {
                "name": "P1",
                "techstack": ["T1", "T2", "T3"],
                "details": ["D1", "D2", "D3"],
            },
            {
                "name": "P2",
                "techstack": ["T1", "T2", "T3"],
                "details": ["D1", "D2", "D3"],
            },
        ]
        template = {
            "complete": "$all_projects",
            "single": "$project_name $techstack $all_details",
            "single-detail": "$detail",
        }

        project_template = ProjectTemplate(template, values)
        return project_template

    def test_render_tex(self, project_obj):
        tex = project_obj.render_tex()
        assert tex == ("P1 T1, T2, T3 \nD1\nD2\nD3\n" "P2 T1, T2, T3 \nD1\nD2\nD3")



import os
import pytest
from resumaker.config import *
from resumaker.utils import *


@pytest.fixture
def delete_config_file():
    if os.path.exists("config.yml"):
        os.remove("config.yml")


@pytest.fixture
def create_invalid_config_file():
    open("config.yml", "w").close()
    yield
    os.remove("config.yml")


@pytest.fixture
def create_valid_config_file():
    f = open("config.yml", "w")
    f.write("key: value")
    f.close()
    yield
    os.remove("config.yml")


class TestGetConfig:
    def test_when_filenotfound(self, mocker):
        patch1 = mocker.patch("resumaker.config.read_file_txt")
        patch1.side_effect = FileNotFoundError
        config = get_config()
        assert config == default_configs

    def test_when_file_exists(self, mocker):
        patch1 = mocker.patch("resumaker.config.read_file_txt")
        patch1.return_value = "hello-world"

        return_obj = {"KEY": "val"}
        patch2 = mocker.patch("resumaker.config.manage_user_config_txt")
        patch2.return_value = return_obj

        config = get_config()
        assert config == return_obj


class TestManageUserConfigTxt:
    def test_when_config_obj_invalid(self, mocker, capsys):
        patch1 = mocker.patch("resumaker.config.render_yaml_txt")
        patch1.return_value = []
        patch2 = mocker.patch("resumaker.config.is_config_valid")
        patch2.return_value = False, "error"

        out = manage_user_config_txt(None)
        captured = capsys.readouterr()

        assert out == default_configs
        assert captured.out == "Warning: error. Using default configs.\n"

    def test_when_config_obj_valid(self, mocker):
        patch1 = mocker.patch("resumaker.config.render_yaml_txt")
        patch1.return_value = True
        patch2 = mocker.patch("resumaker.config.is_config_valid")
        patch2.return_value = True, None

        p3_return = {"KEY": "Merged with user config"}
        patch3 = mocker.patch("resumaker.config.merge_user_config_with_defaults")
        patch3.return_value = p3_return

        out = manage_user_config_txt(None)
        assert out == p3_return


@pytest.fixture
def apply_user_config_obj(request):
    marker = request.node.get_closest_marker("fixture_data")
    if marker is None:
        data = None
    else:
        data = marker.args[0]
    return data


class TestMergeUserConfigWithDefaults:
    @pytest.mark.fixture_data({})
    def test_when_not_overwritten(self, apply_user_config_obj, mocker):
        config = merge_user_config_with_defaults(apply_user_config_obj)
        assert config == default_configs

    @pytest.mark.fixture_data({"RESUME_FILENAME": "custom.yml"})
    def test_when_overwritten_with_string(self, apply_user_config_obj, mocker):
        config = merge_user_config_with_defaults(apply_user_config_obj)
        assert config["RESUME_FILENAME"] == ["custom.yml"]

    @pytest.mark.fixture_data({"RESUME_FILENAME": ["custom.yml"]})
    def test_when_overwritten_with_list(self, apply_user_config_obj, mocker):
        config = merge_user_config_with_defaults(apply_user_config_obj)
        assert config["RESUME_FILENAME"] == ["custom.yml"]

    @pytest.mark.fixture_data({"custom_config": ["custom_value"]})
    def test_ignore_unsupported_config(self, apply_user_config_obj, mocker):
        config = merge_user_config_with_defaults(apply_user_config_obj)
        assert config == default_configs



import os
import pytest
from resumaker.utils import *


class TestReadFileTxt:
    @pytest.fixture
    def create_file(self, request):
        marker = request.node.get_closest_marker("create_file")
        if marker is None:
            data = None
        else:
            filename = marker.args[0]
            txt = marker.args[1]
        f = open(filename, "w")
        f.write(txt)
        f.close()
        yield
        os.remove(filename)

    @pytest.fixture
    def delete_file(self):
        if os.path.exists("file.txt"):
            os.remove("file.txt")

    def test_file_does_not_exist(self, delete_file):
        with pytest.raises(FileNotFoundError):
            read_file_txt("file.txt")

    @pytest.mark.create_file("file.txt", "")
    def test_file_exists_and_empty(self, create_file):
        txt = read_file_txt("file.txt")
        assert txt == ""

    @pytest.mark.create_file("file.txt", "hello-world")
    def test_file_exists_and_not_empty(self, create_file):
        txt = read_file_txt("file.txt")
        assert txt == "hello-world"


class TestIsConfigValid:
    def test_when_dict(self, mocker):
        config = is_config_valid({})
        assert config == (True, None)

    def test_when_list(self, mocker, capsys):
        config = is_config_valid([])
        assert config == (False, "config.yml should be convertible to Python dict")

    def test_when_value_is_dict(self, mocker, capsys):
        config = is_config_valid({"KEY": {}})
        assert config == (False, "Value of config has to be str or list of str")

    def test_when_value_is_list(self, mocker, capsys):
        config = is_config_valid({"KEY": []})
        assert config == (True, None)


class TestIsResumeValid:
    def test_when_dict(self, mocker):
        result = is_resume_obj_valid({}, "resume.yml")
        assert result == (False, "name does not exist in resume.yml")

    def test_when_some_fields_absent(self, mocker):
        result = is_resume_obj_valid(
            {
                "name": "test-user",
                "location": "location-y",
                "contact": {"phone": "9999999"},
                "education": [
                    {"name": "clg-1", "duration": "2222 - 3333", "degree": "deg360"}
                ],
            },
            "resume.yml",
        )
        assert result == (False, "links does not exist in resume.yml")


class TestCommonSectionContact:
    def test_valid(self):
        contact = {"name": "first last"}
        res = is_resume_contact_valid(contact)
        assert res == (True, None)

    def test_invalid(self):
        contact = {"name": ["first"]}
        with pytest.raises(AssertionError) as excinfo:
            res = is_resume_contact_valid(contact)
            assert excinfo.value == (False, "Value of contct.name should be a str")
    
    def test_invalid_key(self):
        contact = {"key": ["first"]}
        with pytest.raises(AssertionError) as excinfo:
            res = is_resume_contact_valid(contact)
            assert excinfo.value == (False, "key is not valid")


class TestCommonSectionEducation:
    def test_valid(self):
        education = [
            {
                "name": "clg",
                "location": "l",
                "duration": "x - y",
                "degree": "degA",
                "achievements": ["a1", "a2"],
            }
        ]
        res = is_resume_education_valid(education)
        assert res == (True, None)

        def test_invalid(self):

            education = [{
            'name': 'clg',
            'location': 'l',
            'duration': 'x - y',
            'degree': "degA",
            'achievements': ['a1', 'a2']
        }]
        res = is_resume_education_valid(education)
        assert res == (True, None)



import sys
import subprocess
import os
import shutil
from pathlib import Path
from string import Template
from resumaker.template_str import template
from resumaker.config import config

BASE_DIR = config["BASE_DIR"]

ORDER = ["summary", "education", "skills", "work_experience", "projects", "links"]


class Resume:
    def __init__(
        self,
        values,
        template,
        /,
        order=ORDER,
    ):
        self.values = values
        self.template = template

        self.contact = Contact()
        self.summary = Summary()
        self.education = Education()
        self.skills = Skills()
        self.work_experience = WorkExperience()
        self.projects = Project()
        self.links = Links()

        self.setup_sections = [
            self.template.get_license(),
            self.template.get_preamble(),
            self.contact.generate_tex(
                self.template,
                self.values["contact"]
                | {"name": self.values["name"]}
                | {"location": self.values["location"]},
            ),
        ]

        # self.internal_sections = [self.summary, self.education, self.skills, self.links]
        self.internal_sections = order if order else ORDER

    def generate_tex(self):
        all_sections = []
        for section in self.setup_sections:
            all_sections.append(section)

        internal_sections = []
        for section in self.internal_sections:
            try:
                tex = getattr(self, section).generate_tex(
                self.template, self.values[section]
            )
            except:
                sys.exit(f"{section} is not valid for template.")
            
            internal_sections.append(tex)

        internal_tex_template = Template(template["main"])
        internal_tex_str = "\n".join(internal_sections)
        rendered_internal_tex = internal_tex_template.substitute(
            CONTENT=internal_tex_str
        )

        all_sections.append(rendered_internal_tex)

        return "\n".join(all_sections)

    def write_tex(self, filepath):
        with open(filepath, "w") as f:
            text = self.generate_tex()
            f.write(text)

    def build(self, filename):
        tex_filename = f"{filename}.tex"
        filepath = BASE_DIR.joinpath(
            f"templates/{self.template.get_template_name()}/{tex_filename}"
        )
        self.write_tex(filepath)

        old_cwd = os.getcwd()
        new_cwd = os.path.dirname(filepath)
        os.chdir(new_cwd)

        return_code = subprocess.run(
            ["pdflatex", "-interaction=batchmode", tex_filename]
            # ["pdflatex", tex_filename]
        )

        pdf_filename = f"{filename}.pdf"
        shutil.move(pdf_filename, os.path.join(old_cwd, pdf_filename))

        for ext in ["log", "out", "tex", "aux"]:
            os.remove(f"{filename}.{ext}")

        os.chdir(old_cwd)

    def __str__(self):
        return f"{self.name}"


class Contact:
    def generate_tex(self, template, values):
        return template.get_tex("contact", values)


class Summary:
    def generate_tex(self, template, values):
        return template.get_tex("summary", values)


class Education:
    def generate_tex(self, template, values):
        return template.get_tex("education", values)


class Links:
    def generate_tex(self, template, values):
        return template.get_tex("links", values)


class Skills:
    def generate_tex(self, template, values):
        return template.get_tex("skills", values)


class WorkExperience:
    def generate_tex(self, template, values):
        return template.get_tex("work_experience", values)


class Project:
    def generate_tex(self, template, values):
        return template.get_tex("projects", values)



import yaml
import sys
from .utils import (
    read_file_txt,
    render_yaml_txt,
)


def get_resume_obj(filenames):
    """
    Return resume dict by rendering first successful filename
    """
    error_messages = []
    for filename in filenames:
        try:
            resume_yaml_txt = read_file_txt(filename)
        except FileNotFoundError:
            error_messages.append(f"{filename} is not found!")
            continue
        resume_obj = render_yaml_txt(resume_yaml_txt)
        return resume_obj, filename, None
    return None, None, error_messages



import pytest
from resumaker.render_resume import get_resume_obj


@pytest.fixture
def create_file(request):
    marker = request.node.get_closest_marker("fixture_data")
    if marker is not None:
        data = marker.args[0]
    else:
        data = None
    return data


class TestGetResumeObj:
    @pytest.mark.fixture_data(["file1.yml"])
    def test_filenames_does_not_exist(self, mocker, create_file):
        patch1 = mocker.patch("resumaker.render_resume.read_file_txt")
        patch1.side_effect = FileNotFoundError

        result, filename, error = get_resume_obj(create_file)
        assert result is None
        assert error is not None
        assert filename == None
        assert error[0] == "file1.yml is not found!"

    @pytest.mark.fixture_data(["file1.yml"])
    def test_filenames_exists_but_invalid(self, mocker, create_file):
        patch1 = mocker.patch("resumaker.render_resume.read_file_txt")
        patch1.return_value = "%sk lsdjf ls"

        patch1 = mocker.patch("resumaker.render_resume.render_yaml_txt")
        patch1.side_effect = SystemExit

        with pytest.raises(SystemExit):
            result,filename, error = get_resume_obj(create_file)

    @pytest.mark.fixture_data(["file1.yml"])
    def test_filenames_exists_and_valid(self, mocker, create_file):
        patch1 = mocker.patch("resumaker.render_resume.read_file_txt")
        patch1.return_value = "key: val"

        patch1 = mocker.patch("resumaker.render_resume.render_yaml_txt")
        patch1.return_value = {"key": "val"}

        result, filename,  error = get_resume_obj(create_file)
        assert result is not None
        assert error is None
        assert filename == "file1.yml"
        assert result == {"key": "val"}



import json
import os
from string import Template
from resumaker.config import BASE_DIR
from resumaker.utils import render_yaml_txt


class ResumeTemplate:
    "An interface between resume and actual template string."

    def __init__(self, name):
        self.name = name
        self.tex_template_obj = self.get_template_obj()

        self.contact = ContactTemplate
        self.summary = SummaryTemplate
        self.education = EducationTemplate
        self.skills = SkillsTemplate
        self.work_experience = WorkExperienceTemplate
        self.projects = ProjectTemplate
        self.links = LinksTemplate

    def get_template_name(self):
        return self.name

    def get_template_obj(self):
        with open(
            os.path.join(BASE_DIR, "templates", self.name, "tex_template.yml")
        ) as f:
            tex = f.read()

            tex_obj = render_yaml_txt(tex)
            return tex_obj

    def get_tex(self, section, values):
        return getattr(self, section)(
            self.tex_template_obj[section], values
        ).render_tex()

    def get_preamble(self):
        return self.tex_template_obj["preamble"]

    def get_license(self):
        return self.tex_template_obj["license"]


class ResumeSectionTemplate:
    def __init__(self, template, values):
        self.values = values
        self.template = template


class ContactTemplate(ResumeSectionTemplate):
    def render_tex(self):
        pyTemplate = Template(self.template)
        result = pyTemplate.substitute(
            name=self.values["name"],
            location=self.values["location"],
            phone=self.values["phone"],
            email=self.values["email"],
            linkedin=self.values["linkedin"],
            github=self.values["github"],
        )
        return result


class SummaryTemplate(ResumeSectionTemplate):
    def render_tex(self):
        pyTemplate = Template(self.template)
        result = pyTemplate.substitute(
            title=self.values["title"],
            text=self.values["text"],
        )
        return result


class EducationTemplate(ResumeSectionTemplate):
    def render_tex(self):
        py_complete_template = Template(self.template["complete"])
        py_single_template = Template(self.template["single"])
        py_achievement_template = Template(self.template["achievement"])

        rendered_single_educations = []
        for institute in self.values:
            rendered_achievements = []
            for achievement in institute["achievements"]:
                rendered_achievement = py_achievement_template.substitute(
                    achievement=achievement
                )
                rendered_achievements.append(rendered_achievement)
            rendered_single_education = py_single_template.substitute(
                name=institute["name"],
                duration=institute["duration"],
                degree=institute["degree"],
                location=institute["location"],
                all_achievements=" ".join(rendered_achievements),
            )
            rendered_single_educations.append(rendered_single_education)

        all_educations = "\n".join(rendered_single_educations)
        rendered_complete_education = py_complete_template.substitute(
            all_educations=all_educations
        )
        return rendered_complete_education


class LinksTemplate(ResumeSectionTemplate):
    def render_tex(self):
        py_complete_template = Template(self.template["complete"])
        py_single_template = Template(self.template["single"])
        py_single_link_template = Template(self.template["single-link"])

        rendered_links = []
        for link_category, link_lists in self.values.items():
            single_rendered_link = []
            for link in link_lists:
                single_rendered_link.append(
                    py_single_link_template.substitute(
                        link_url=link["url"], link_url_text=link["url_text"]
                    )
                )
            rendered_links.append(
                py_single_template.substitute(
                    link_title=" ".join(
                        list(
                            map(
                                lambda x: x[0].upper() + x[1:], link_category.split("-")
                            )
                        )
                    ),
                    single_links=" ".join(single_rendered_link),
                )
            )
        all_links = py_complete_template.substitute(all_links="\n".join(rendered_links))
        return all_links


class SkillsTemplate(ResumeSectionTemplate):
    def render_tex(self):
        complete_template = Template(self.template["complete"])
        single_template = Template(self.template["single"])
        rendered_skill_topics = []
        for skill_topic, skills_list in self.values.items():
            rendered_skill_topics.append(
                single_template.substitute(
                    skill_topic=skill_topic[0].upper() + skill_topic[1:],
                    skills_list=self.format_skills_list(skills_list),
                )
            )
        all_skills = complete_template.substitute(
            all_skills="\n".join(rendered_skill_topics)
        )
        return all_skills

    def format_skills_list(self, skills: list) -> str:
        line_width = -2
        lines = []
        line = []

        for skill in skills:
            if line_width + len(skill) > 100:
                lines.append(line[:])
                line = []
                line_width = -2
            line.append(skill)
            line_width += len(skill) + 2

        lines.append(line)
        lines = list(map(lambda x: ", ".join(x), lines))
        result = " \\\\ &\n".join(lines)
        return result


class WorkExperienceTemplate(ResumeSectionTemplate):
    def render_tex(self):
        complete_template = Template(self.template["complete"])
        single_template = Template(self.template["single"])
        contribution_template = Template(self.template["single-contribution"])

        rendered_single = []
        for experience in self.values:
            company_name = experience["company-name"]
            company_location = experience["location"]
            position = experience["role"]
            duration = experience["duration"]

            rendered_contributions = []
            for contribution in experience["contributions"]:
                rendered_contributions.append(
                    contribution_template.substitute(contribution=contribution)
                )

            rendered_single.append(
                single_template.substitute(
                    company_name=company_name,
                    location=company_location,
                    role=position,
                    duration=duration,
                    all_contributions="\n".join(rendered_contributions),
                )
            )

        all_experiences = complete_template.substitute(
            all_work_experiences="\n".join(rendered_single)
        )

        return all_experiences


class ProjectTemplate(ResumeSectionTemplate):
    def render_tex(self):
        complete_template = Template(self.template["complete"])
        single_template = Template(self.template["single"])
        detail_template = Template(self.template["single-detail"])

        rendered_single = []
        for project in self.values:
            project_name = project["name"]
            techstack = ", ".join(project["techstack"])

            rendered_details = []
            for detail in project["details"]:
                rendered_details.append(detail_template.substitute(detail=detail))

            rendered_single.append(
                single_template.substitute(
                    project_name=project_name,
                    techstack=techstack,
                    all_details="\n" + "\n".join(rendered_details),
                )
            )

        all_projects = complete_template.substitute(
            all_projects="\n".join(rendered_single)
        )

        return all_projects



tex_template = {
    "license": r"""
% Medium Length Professional CV
% LaTeX Template
% Version 3.0 (December 17, 2022)
%
% This template originates from:
% https://www.LaTeXTemplates.com
%
% Author:
% Vel (vel@latextemplates.com)
%
% Original author:
% Trey Hunner (http://www.treyhunner.com/)
%
% License:
% CC BY-NC-SA 4.0 (https://creativecommons.org/licenses/by-nc-sa/4.0/)
% FILE HAS BEEN MODIFIED
""",
    "setup": r"""
\documentclass[
	%a4paper, % Uncomment for A4 paper size (default is US letter)
	11pt, % Default font size, can use 10pt, 11pt or 12pt
]{resume} % Use the resume class

\usepackage{ebgaramond} % Use the EB Garamond font
\usepackage{hyperref}
\hypersetup
{
    colorlinks=true,
    urlcolor=blue
}

\input{glyphtounicode}
\pdfgentounicode=1
""",
    "main": r"""
\begin{document}
$CONTENT
\end{document}
""",
    "contact": r"""
\name{$name}
\address{$location}
\address{\raisebox{-2px}{\includegraphics[width=10px]{icons/phone.png}}  $phone \\ \raisebox{-2px}{\includegraphics[width=10px]{icons/mail.png}} $email  \\ \raisebox{-2px}{\includegraphics[width=10px]{icons/linkedin.png}} \href{https://www.linkedin.com/in/$linkedin/}{linkedin.com/in/$linkedin} \\ \raisebox{-2px}{\includegraphics[width=10px]{icons/github.png}} \href{https://github.com/$github}{github.com/$github}}
""",
    "education": {
        "complete": r"""
    \begin{rSection}{Education}
        $all_education
    \end{rSection}""",
        "single": r"""

        \textbf{$degree} \hfill \textit{$duration} \\ 
        $institution_name, $institution_location \\
        $all_achievements
        """,
        "achievement": r"$single_achievement \\",
    },
    "links": {
        "complete": r"""
\begin{rSection}{Links}
	\begin{tabular}{@{} >{\bfseries}l @{\hspace{6ex}} l @{}}
        $all_links
	\end{tabular}
\end{rSection}""",
        "single": r"$link_title & \href{$link_url}{$link_url_text} \\",
    },
    "summary": r"""
\begin{rSection}{$title}
    $text
\end{rSection}
""",
    "skills": {
        "complete": r"""
            \begin{rSection}{Skills}
                \begin{tabular}{@{} >{\bfseries}l @{\hspace{6ex}} l @{}}
                    $all_skills
                \end{tabular}
            \end{rSection}
    """,
        "single": r"$skill_topic & $skills_list \\",
    },
    "work_experience": {
        "complete": r"""
                \begin{rSection}{Work Experience}
                \def \\ {\addressSep\ } 
                $all_work_experiences
            \end{rSection}
    """,
        "single": r"""
            \begin{rSubsection}{$company_name, $company_location \\ $position \\ $duration}{}{}{}
                    $all_contributions
	        \end{rSubsection}
        """,
        "single-contribution": r"\item $contribution",
    },
    "projects": {
        "complete": r"""
                \begin{rSection}{Projects}
                $all_projects
            \end{rSection}
    """,
        "single": r"""
            \begin{rSubsection}{$project_name}{$technologies}{}{}
                    $all_details
	        \end{rSubsection}
        """,
        "single-detail": r"\item $detail",
    },
}



"""
Resumaker
---------

Effortlessly create multiple, ATS-friendly resumes from a single YAML file with Resumaker.

Resumaker is a powerful and efficient tool designed to streamline the resume
creation process for professionals. By leveraging a single YAML file, Resumaker
generates multiple, professionally formatted, and ATS-friendly resumes tailored
to different job applications. This innovative solution eliminates the need for
manual formatting and structuring, allowing you to focus on the content and
details of your resume.

Github: https://github.com/sujaudd1n/resumaker
"""

from resumaker.__about__ import __version__, __license__, __author__
